import { Disposable, Listener, TypedEvent } from "../helpers/typed_event";
import { GetRootDirUseCase } from "../usecases/get_root_dir_usecase"
import { exec } from 'child_process';

export const activeProcessPids: { [name: string]: { pid: number, status: ProcessStatus } } = {}

export enum ProcessStatus {
    run = 'run',
    endError = 'endError',
    endOk = 'endOk',
    userDelete = 'userDelete',
}

class ExecutorProgramServiceV2 extends TypedEvent<{ [name: string]: { pid: number, status: ProcessStatus } }> {}

export const executorProgramServiceV2 = new ExecutorProgramServiceV2();
class ProcessData {
    log: string | undefined;
    status: ProcessStatus;
    constructor(log: string | undefined, status: ProcessStatus) {

        if (log !== undefined) {
            this.log = log;
        }
        this.status = status;

    }
}
abstract class Watcher {
}
export abstract class ExecProcessWatcher extends TypedEvent<ProcessData> implements Watcher {
    logs: string[] = [];
    status: ProcessStatus;
    constructor() {
        super();
        this.on((event) => {
            if (event.log !== undefined) {
                this.logs.push(event.log);
            }
            this.status = event.status;
            if (event.status.isEqualMany([ProcessStatus.endError, ProcessStatus.endOk])) {
                this.result()
            }
        })
    }
    abstract result(): Promise<any>

}
export class ExecProcessScenarioV2 {
    call = (command: string, watcher: ExecProcessWatcher, name?: string): void => {
        const process = exec(command, { cwd: new GetRootDirUseCase().call() });
        if (process.pid) {
            activeProcessPids[name ?? command] = { pid: process.pid, status: ProcessStatus.run };
            executorProgramServiceV2.emit(activeProcessPids);
        }
        process.stdout.on('data', (data) => watcher.emit(new ProcessData(data, ProcessStatus.run)))
        process.stderr.on('data', (data) => watcher.emit(new ProcessData(data, ProcessStatus.run)));
        process.on('close', (code) => {
            if (code === 0) {
                watcher.emit(new ProcessData(undefined, ProcessStatus.endOk))
                activeProcessPids[name ?? command] = { pid: process.pid ?? 0, status: ProcessStatus.endOk }
                executorProgramServiceV2.emit(activeProcessPids);
            } else {
                watcher.emit(new ProcessData(undefined, ProcessStatus.endError))
                activeProcessPids[name ?? command] = { pid: process.pid ?? 0, status: ProcessStatus.endError };
                executorProgramServiceV2.emit(activeProcessPids);
            }
        });

        process.on('error', (err) => watcher.emit(new ProcessData(err.message, ProcessStatus.endError)));
    }
}