import { ObjectId } from "mongoose";
import { CallbackStrategyWithIdQuery, ResponseBase } from "../../../core/controllers/http_controller";
import { ExecProcessUseCase, IsHaveActiveProcessUseCase } from "../../../core/usecases/exec_process_usecase";
import { ReadByIdDataBaseModelUseCase } from "../../../core/usecases/read_by_id_database_model_usecase";
import { MongoIdValidation } from "../../../core/validations/mongo_id_validation";
import { ProcessWatcherAndDatabaseUpdateService } from "../../datasets/domain/create_dataset_scenario";
import { CalculationInstanceDBModel, ICalculationInstance } from "../models/calculations_instance_database_model";
import { Result } from "../../../core/helpers/result";
import { CreateFileUseCase } from "../../../core/usecases/create_file_usecase";
import { ExecProcessScenarioV2, ExecProcessWatcher } from "../../../core/scenarios/exec_process_scenario_v2";
class ExecProcess extends ExecProcessWatcher {
  id: string;
  constructor(id: string) {
    super()
    this.id = id;
  }
  result = async (): Promise<any> => (await new ReadByIdDataBaseModelUseCase<ICalculationInstance>(CalculationInstanceDBModel).call(this.id)).map(async (model) => {
    model.lastProcessLogs = this.logs.join('\n')
    model.processStatus = this.status;
    // @ts-ignore
    await model.save()
  });

}
export class ExecCalculationInstanceProcessScenario extends CallbackStrategyWithIdQuery {
  idValidationExpression = new MongoIdValidation();
  call = async (id: string): ResponseBase =>
    (await new ReadByIdDataBaseModelUseCase<ICalculationInstance>(CalculationInstanceDBModel).call(id)).map(
      async (model) => {
        const fileOutPath = model.instancePath.pathNormalize() + "/form.json";
        return (await new IsHaveActiveProcessUseCase().call()).map(async () => {
          const execCommand = `${model.script
            } --path ${model.instancePath.pathNormalize()} --form ${fileOutPath}`.replace("\n", "");

          await new CreateFileUseCase().call(fileOutPath, Buffer.from(JSON.stringify(model.formBuilder)));
          await CalculationInstanceDBModel.findById(id).updateOne({
            processStatus: "RUN",
            lastProcessExecCommand: execCommand,
          });
          
          new ExecProcessScenarioV2().call(execCommand, new ExecProcess(model._id))

          return Result.ok("OK");
        });
      }
    );
}
