import {
  CallbackStrategyWithIdQuery,
  CallbackStrategyWithValidationModel,
  ResponseBase,
} from "../../../core/controllers/http_controller";
import { StaticFilesProject } from "../../../core/models/static_files";
import {
  ExecProcessScenarioV2,
  ExecProcessWatcher,
  activeProcessPids,
} from "../../../core/scenarios/exec_process_scenario_v2";
import { ReadByIdDataBaseModelScenario } from "../../../core/scenarios/read_by_id_database_model_scenario";
import { SearchOneDataBaseModelUseCase } from "../../../core/usecases/search_database_model_usecase";
import { CoreValidation } from "../../../core/validations/core_validation";
import { MongoIdValidation } from "../../../core/validations/mongo_id_validation";
import { BehaviorTreeDBModel } from "../../behavior_trees/models/behavior_tree_database_model";
import { BehaviorTreeValidationModel } from "../../behavior_trees/models/behavior_tree_validation_model";
import { IProjectModel, ProjectDBModel } from "../../projects/models/project_model_database_model";
import { GetCommandScenario } from "./get_command_scenario";

class ProcessWatcher extends ExecProcessWatcher {
  async result(): Promise<any> {
    console.log(this);
  }
}

export enum Proceed {
  EXEC_BT = "EXEC_BT",
}
export class ExecBtScenario extends CallbackStrategyWithIdQuery {
  idValidationExpression: CoreValidation = new MongoIdValidation();
  call = async (id: string): ResponseBase =>
    (await new ReadByIdDataBaseModelScenario<BehaviorTreeValidationModel>(BehaviorTreeDBModel).call(id)).map(
      async (behaviorTreeValidationModel) =>
        (
          await new SearchOneDataBaseModelUseCase<IProjectModel>(ProjectDBModel).call(
            { isActive: true },
            "no active projects"
          )
        ).map(async (model) =>
          (await new GetCommandScenario().call("btRuntimeProcess")).map((execProcessData) =>
            new ExecProcessScenarioV2().call(
              execProcessData.execCommand.replace(
                "${bt_path}",
                `${model.rootDir}/${StaticFilesProject.behaviorTrees}/${behaviorTreeValidationModel.name}/bt.xml`
              ),
              new ProcessWatcher(),
              Proceed.EXEC_BT
            )
          )
        )
    );
}
