import { IsString } from "class-validator";
import { CallbackStrategyWithValidationModel, ResponseBase } from "../../../core/controllers/http_controller";
import { ProcessStatus, activeProcessPids } from "../../../core/scenarios/exec_process_scenario_v2";
import { Result } from "../../../core/helpers/result";
import { GetRootDirUseCase } from "../../../core/usecases/get_root_dir_usecase";
import { exec } from 'child_process';


export class StopProcessModel {
    @IsString()
    pid: string
}

export class StopProcessUseCase extends CallbackStrategyWithValidationModel<StopProcessModel> {
    validationModel: StopProcessModel = new StopProcessModel();
    call = async (model: StopProcessModel): ResponseBase => {
        try {
            if (activeProcessPids[model.pid] === undefined) {
                return Result.error('missing pid');
            }
            const processKillStatus = await new Promise<string>((resolve, reject) => {
                try {
                    exec(`kill ${activeProcessPids[model.pid].pid}`, { cwd: new GetRootDirUseCase().call() }, (error, stdout, stderr) => {
                        if (error) {
                            reject(`Ошибка: ${stderr}`); return;
                        }
                        if (stderr) {
                            reject(`Ошибка: ${stderr}`);
                            return;
                        }

                        resolve('Process kill')

                    });
                } catch (e) {
                    resolve('Process kill')
                }

            })
            if (processKillStatus == 'Process kill') {
                activeProcessPids[model.pid].status = ProcessStatus.userDelete;
            }
            return Result.ok(processKillStatus);
        } catch (error) {
            return Result.ok(error)
        }


    }

}